###############################################################################
#
#                              C O N Q S T R A T
#
#            Copyright (C)1985-1986 by Jef Poskanzer and Craig Leres
#
#    Permission to use, copy, modify, and distribute this software and
#    its documentation for any purpose and without fee is hereby granted,
#    provided that this copyright notice appear in all copies and in all
#    supporting documentation. Jef Poskanzer and Craig Leres make no
#    representations about the suitability of this software for any
#    purpose. It is provided "as is" without express or implied warranty.
#
###############################################################################
#
#	Detailed revision history lives in "incl/conqdef"
#
###############################################################################

%{
### yyparse - yacc parser for conqstrat
#
NOIMPLICIT

    integer invertop
    character buf(MAXLINE)
    CONQSTRATCOMMON
%}

%token TOK_VARIABLE
%token TOK_OPERATOR
%token TOK_NUMBER
%token TOK_TERMINATOR
%token TOK_ACTION
%token TOK_AND
%token TOK_NOT

%%

rulelist	:	rulelist rule
		|	rule
		;

rule		:	exprlist TOK_TERMINATOR TOK_ACTION
			%{
			call dumprule( $3 )
			%}
		|	TOK_TERMINATOR TOK_ACTION
			%{
			call dumprule( $2 )
			%}
		|	error
		;

exprlist	:	expr
		|	exprlist TOK_AND expr
		;

expr		:	TOK_VARIABLE TOK_OPERATOR TOK_NUMBER
			%{
			call addrule( $1, $2, $3 )
			%}
		|	TOK_NUMBER TOK_OPERATOR TOK_VARIABLE
			%{
			call addrule( $3, invertop( $2 ), $1 )
			%}
		|	TOK_VARIABLE
			%{
			call addrule( $1, OP_NE, 0 )
			%}
		|	TOK_NOT TOK_VARIABLE
			%{
			call addrule( $2, OP_EQ, 0 )
			%}
		;

%%

%{
### yylex - lexical scanner for conqstrat
#

# Global defines go here.
include "conqdef"

#define(DEBUG,)		# uncomment this to get extra debug code

define(OP_LT,101)
define(OP_LE,102)
define(OP_GT,103)
define(OP_GE,104)
define(OP_EQ,105)
define(OP_NE,106)

define(CONQSTRATCOMMON,
    integer fd, line, rulenum, trstrat(MAX_VAR,10), trvec(32)
    logical goterror, debug, verbose
    character filename(MAXLINE)
    common /conqstratcommon/ fd, line, rulenum, trstrat, trvec,
	goterror, debug, verbose, filename)

define(YYINPUT,
    {line = line + 1
    $3 = getlin( $1($2), fd )})

define(YYDEFAULTACTION,
    {YYGETTEXT( buf )
    call eprintf( "conqstrat: Bad input at line %d, @"%s@"@n", line, buf )
    goterror = .true.})

define(YYERRACTION,call myerrmsg( $1 ))

#define(YYTRACE,call eprintf( " tok %d, (val %d)@n", yytok, yylexval ))

define(RET_VARIABLE,{yyval = $1; return( TOK_VARIABLE )})
define(RET_ACTION,{yyval = $1; return( TOK_ACTION )})
define(RET_OPERATOR,{yyval = $1; return( TOK_OPERATOR )})
%}

WS	[ @t]+
OWS	[ @t]*

%%

    NOIMPLICIT
    character buf(MAXLINE)
    integer i, ctoi
    CONQSTRATCOMMON

    yyval = 0

random			RET_VARIABLE( VAR_RANDOM )
dne			RET_VARIABLE( VAR_DNE )
damage			RET_VARIABLE( VAR_DAMAGE )
incoming		RET_VARIABLE( VAR_INCOMING )
fuel			RET_VARIABLE( VAR_FUEL )
numtorps		RET_VARIABLE( VAR_NUMTORPS )
shields			RET_VARIABLE( VAR_SHIELDS )
etemp			RET_VARIABLE( VAR_ETEMP )
wtemp			RET_VARIABLE( VAR_WTEMP )
phaserdam		RET_VARIABLE( VAR_PHASERDAM )
torpdam			RET_VARIABLE( VAR_TORPDAM )
warp			RET_VARIABLE( VAR_WARP )
shup			RET_VARIABLE( VAR_SHUP )
walloc			RET_VARIABLE( VAR_WALLOC )
orbiting		RET_VARIABLE( VAR_ORBITING )
repairing		RET_VARIABLE( VAR_REPAIRING )
cloaked			RET_VARIABLE( VAR_CLOAKED )
enemycloaked		RET_VARIABLE( VAR_ENEMYCLOAKED )
enemydamage		RET_VARIABLE( VAR_ENEMYDAMAGE )
canread			RET_VARIABLE( VAR_CANREAD )

rob_noop		RET_ACTION( ROB_NOOP )
rob_gohome		RET_ACTION( ROB_GOHOME )
rob_gorepair		RET_ACTION( ROB_GOREPAIR )
rob_allocate		RET_ACTION( ROB_ALLOCATE )
rob_detonate		RET_ACTION( ROB_DETONATE )
rob_mydetonate		RET_ACTION( ROB_MYDETONATE )
rob_phaser		RET_ACTION( ROB_PHASER )
rob_torpedo		RET_ACTION( ROB_TORPEDO )
rob_burst		RET_ACTION( ROB_BURST )
rob_shield		RET_ACTION( ROB_SHIELD )
rob_untractor		RET_ACTION( ROB_UNTRACTOR )
rob_warp_0		RET_ACTION( ROB_WARP_0 )
rob_warp_2		RET_ACTION( ROB_WARP_2 )
rob_warp_5		RET_ACTION( ROB_WARP_5 )
rob_warp_8		RET_ACTION( ROB_WARP_8 )
rob_track		RET_ACTION( ROB_TRACK )
rob_slient		RET_ACTION( ROB_SLIENT )
rob_message		RET_ACTION( ROB_MESSAGE )
rob_takedrugs		RET_ACTION( ROB_TAKEDRUGS )
rob_repair		RET_ACTION( ROB_REPAIR )
rob_readmsg		RET_ACTION( ROB_READMSG )
rob_insult		RET_ACTION( ROB_INSULT )
rob_gofuel		RET_ACTION( ROB_GOFUEL )
rob_runaway		RET_ACTION( ROB_RUNAWAY )

"<"			RET_OPERATOR( OP_LT )
"<="			RET_OPERATOR( OP_LE )
">"			RET_OPERATOR( OP_GT )
">="			RET_OPERATOR( OP_GE )
"="|"=="		RET_OPERATOR( OP_EQ )
"!="|"<>"		RET_OPERATOR( OP_NE )

"!"			return( TOK_NOT )
"&"			return( TOK_AND )
";"			return( TOK_TERMINATOR )

[0-9]			%{
			YYGETTEXT( buf )
			i = 1
			yyval = ctoi( buf, i )
			return( TOK_NUMBER )
			%}

#?*			# eat comments

{WS}			# eat gratuitous white space

@n			# eat newlines

%%

### conqstrat - main program
#
DRIVER(conqstrat)
NOIMPLICIT

    integer i, getarg, open, yyparse
    character buf(MAXLINE)
    CONQSTRATCOMMON
    string usage "usage: conqstrat [-vd] [file]"

    debug = .false.
    verbose = .false.

    while ( getarg( 1, buf, MAXLINE ) != EOF )
	{
	if ( buf(1) != '-' )
	    break
	for ( i = 2; buf(i) != EOS; i = i + 1 )
	    switch ( buf(i) )
		{
		case 'd', 'D':
		    debug = .true.
		case 'v', 'V':
		    verbose = .true.
		default:
		    call error( usage )
		}
	call delarg( 1 )
	}

    if ( getarg( 1, filename, FILENAMESIZE ) == EOF )
	{
	call strcpy( "<STDIN>", filename )
	fd = STDIN
	}
    else
	{
	call delarg( 1 )
	if ( getarg( 1, buf, MAXLINE ) != EOF )
	    call error( usage )
	fd = open( filename, READ )
	if ( fd == ERR )
	    {
	    call eprintf( "conqstrat: %s - can't open", filename )
	    call error( "" )
	    }
	}

    call initrun

    if ( yyparse( i ) == ERR )
	call error( "conqstrat: yyparse() error" )
    if ( goterror )
	call error( "conqstrat: got error" )

    if ( fd != STDIN )
	call close( fd )

    if ( verbose )
	call displayrun

    call dumprun

DRETURN

end


### initrun - initalize for the run
#
# synopsis
#
#    call initrun
#
subroutine initrun
NOIMPLICIT

    integer i, j
    CONQSTRATCOMMON

    # Initialize random stuff.
    rulenum = 1
    line = 0
    goterror = .false.

    # Initialize the strategy table to accept everything.
    for ( i = 1; i <= MAX_VAR; i = i + 1 )
	for ( j = 1; j <= 10; j = j + 1 )
	    trstrat(i,j) = -1		# set all bits

    # Initialize the rule to action vector to nothing.
    for ( i = 1; i <= 32; i = i + 1 )
	trvec(i) = ROB_NOOP

    return

end


### dumprun - dump the run into the common block
#
# synopsis
#
#    call dumprun
#
subroutine dumprun
NOIMPLICIT

    integer i, j
    CONQSTRATCOMMON
    include "conqcom"

    # This is the only place we include/modify the shared common block.
    if ( commonrev != COMMONSTAMP )
	call error ( "conqstrat: Common block ident mismatch." )

    # Copy the strategy table.
    for ( i = 1; i <= MAX_VAR; i = i + 1 )
	for ( j = 1; j <= 10; j = j + 1 )
	    rstrat(i,j) = trstrat(i,j)

    # Copy the action vector.
    for ( i = 1; i <= 32; i = i + 1 )
	rvec(i) = trvec(i)

    return

end


### displayrun - dump the run to STDOUT
#
# synopsis
#
#    call displayrun
#
subroutine displayrun
NOIMPLICIT

    integer i, j
    character buf(MAXLINE)
    CONQSTRATCOMMON
    string strat "trstrat"
    string vec "trvec"

    call getdandt( buf )
    call printf( "# conqdata - robot strategy data generated on %s@n", buf )
    call printf( "    integer %s(%d,10), %s(32)@n", strat, MAX_VAR, vec )

    call printf( "    data %s /", strat )
    for ( j = 1; j <= 10; j = j + 1 )
	for ( i = 1; i <= MAX_VAR; i = i + 1 )
	    {
	    call printf( "%d", trstrat(i,j) )
	    if ( j < 10 | i < MAX_VAR )
		{
		if ( mod( i, 5 ) == 0 )
		    call puts( ",@n@t" )
		else
		    call putc( ',' )
		}
	    }
    call puts( "/@n" )

    call printf( "    data %s /", vec )
    for ( i = 1; i <= 32; i = i + 1 )
	{
	call printf( "%d", trvec(i) )
	if ( i == 16 )
	    call puts( ",@n@t" )
	else if ( i < 32 )
	    call putc( ',' )
	}
    call puts( "/@n" )

    return

end


### dumprule - dump the current rule
#
# synopsis
#
#    integer action
#    call dumprule( action )
#
subroutine dumprule( action )
NOIMPLICIT
integer action

    integer i, j, tbits
    character buf(MAXLINE)
    CONQSTRATCOMMON

    # Store action.
    trvec(rulenum) = action

    # Check for impossible rules.
    for ( i = 1; i <= MAX_VAR; i = i + 1 )
	{
	# There must be at least one value that this variable accepts.
	tbits = 0
	for ( j = 1; j <= 10; j = j + 1 )
	    tbits = ior( tbits, trstrat(i,j) )
	if ( iand( tbits, ibset( 0, rulenum - 1 ) ) == 0 )
	    {
	    call valstr( i, buf )
	    call eprintf(
		"conqstrat: Rule on line %d can't happen, %s is broken@n",
		line - 1, buf)
	    }
	}

    # Update rule number.
    rulenum = rulenum + 1

    return

end


### addrule - add info to the current rule
#
# synopsis
#
#    integer var, op, num
#    call addrule( var, op, num )
#
subroutine addrule( var, op, num )
NOIMPLICIT
integer var, op, num

    integer i, rulebits
    character svar(32), sop(32)
    CONQSTRATCOMMON

    if ( debug )
	{
	call valstr( var, svar )
	call valstr( op, sop )
	call eprintf( "addrule: var %s, op %s, num %d@n", svar, sop, num )
	}

    if ( rulenum > 32 )
	BAIL( "conqstrat: More than 32 rules; line %d@n", line )
    if ( num < 0 | num > 9 )
	BAIL( "addrule: impossible number %d@n", num )
    if ( var < 1 | num > MAX_VAR )
	BAIL( "addrule: impossible variable %d@n", var )

    rulebits = not( ibset( 0, rulenum - 1 ) )
    switch ( op )
	{
	case OP_LT:
	    for ( i = num; i <= 9; i = i + 1 )
		trstrat(var,i+1) = iand( trstrat(var,i+1), rulebits )
	case OP_LE:
	    for ( i = num + 1; i <= 9; i = i + 1 )
		trstrat(var,i+1) = iand( trstrat(var,i+1), rulebits )
	case OP_GT:
	    for ( i = 0; i <= num; i = i + 1 )
		trstrat(var,i+1) = iand( trstrat(var,i+1), rulebits )
	case OP_GE:
	    for ( i = 0; i < num; i = i + 1 )
		trstrat(var,i+1) = iand( trstrat(var,i+1), rulebits )
	case OP_EQ:
	    for ( i = 0; i <= 9; i = i + 1 )
		if ( i != num )
		    trstrat(var,i+1) = iand( trstrat(var,i+1), rulebits )
	case OP_NE:
	    trstrat(var,num+1) = iand( trstrat(var,num+1), rulebits )
	default:
	    BAIL( "addrule: impossible op %d@n", op )
	}

    return

end


### invertop - invert an operator
#
# synopsis
#
#    integer iop, op, invertop
#    iop = invertop( op )
#
integer function invertop( op )
NOIMPLICIT
integer op

    integer iop

    switch ( op )
	{
	case OP_LT:
	    iop = OP_GE
	case OP_LE:
	    iop = OP_GT
	case OP_GT:
	    iop = OP_LE
	case OP_GE:
	    iop = OP_LT
	default:
	    iop = op
	}

    return ( iop )

end


### valstr - convert a var/op value to a string
#
# synopsis
#
#    integer value
#    character buf()
#    call valstr( value, buf )
#
subroutine valstr( value, buf )
NOIMPLICIT
integer value
character buf(ARB)

    switch ( value )
	{
	case VAR_RANDOM:
	    call strcpy( "random", buf )
	case VAR_DNE:
	    call strcpy( "dne", buf )
	case VAR_DAMAGE:
	    call strcpy( "damage", buf )
	case VAR_INCOMING:
	    call strcpy( "incoming", buf )
	case VAR_FUEL:
	    call strcpy( "fuel", buf )
	case VAR_NUMTORPS:
	    call strcpy( "numtorps", buf )
	case VAR_SHIELDS:
	    call strcpy( "shields", buf )
	case VAR_ETEMP:
	    call strcpy( "etemp", buf )
	case VAR_WTEMP:
	    call strcpy( "wtemp", buf )
	case VAR_PHASERDAM:
	    call strcpy( "phaserdam", buf )
	case VAR_TORPDAM:
	    call strcpy( "torpdam", buf )
	case VAR_WARP:
	    call strcpy( "warp", buf )
	case VAR_SHUP:
	    call strcpy( "shup", buf )
	case VAR_WALLOC:
	    call strcpy( "walloc", buf )
	case VAR_ORBITING:
	    call strcpy( "orbiting", buf )
	case VAR_REPAIRING:
	    call strcpy( "repairing", buf )
	case VAR_CLOAKED:
	    call strcpy( "cloaked", buf )
	case VAR_ENEMYCLOAKED:
	    call strcpy( "enemycloaked", buf )
	case VAR_ENEMYDAMAGE:
	    call strcpy( "enemydamage", buf )
	case VAR_CANREAD:
	    call strcpy( "canread", buf )
	case OP_LT:
	    call strcpy( "lt", buf )
	case OP_LE:
	    call strcpy( "le", buf )
	case OP_GT:
	    call strcpy( "gt", buf )
	case OP_GE:
	    call strcpy( "ge", buf )
	case OP_EQ:
	    call strcpy( "eq", buf )
	case OP_NE:
	    call strcpy( "ne", buf )
	default:
	    call prints( buf, "<%d>", value )
	}

    return

end

### tokstr - convert a token to a string
#
# synopsis
#
#    integer token
#    character buf()
#    call tokstr( token, buf )
#
subroutine tokstr( token, buf )
NOIMPLICIT
integer token
character buf(ARB)

    switch ( token )
	{
	case TOK_VARIABLE:
	    call strcpy( "TOK_VARIABLE", buf )
	case TOK_OPERATOR:
	    call strcpy( "TOK_OPERATOR", buf )
	case TOK_NUMBER:
	    call strcpy( "TOK_NUMBER", buf )
	case TOK_TERMINATOR:
	    call strcpy( "TOK_TERMINATOR", buf )
	case TOK_ACTION:
	    call strcpy( "TOK_ACTION", buf )
	case TOK_AND:
	    call strcpy( "TOK_AND", buf )
	case TOK_NOT:
	    call strcpy( "TOK_NOT", buf )
	case YYENDTOK:
	    call strcpy( "YYENDTOK", buf )
	default:
	    call prints( buf, "TOK_<%d>", token )
	}

    return

end

### myerrmsg - yacc error reporting routine
#
# synopsis
#
#    character buf()
#    call myerrmsg( buf )
#
subroutine myerrmsg( buf )
NOIMPLICIT
character buf(ARB)

    CONQSTRATCOMMON

    call eprintf( "conqstrat: error at line %d: %s@n", line, buf )
    goterror = .true.

end
